import { DATA_FORMAT, DATA_TYPE, REPORT_GROUP } from '../../../constant';
import { ReportConfigEntity } from '../../../entities/report-config.entity';

export default <ReportConfigEntity>{
  group_name: REPORT_GROUP.transaction_report,
  unique_name: `${REPORT_GROUP.transaction_report}__refunds`,
  label: 'Pengembalian',
  table_schema: `refunds AS main
  LEFT JOIN transactions tr ON tr.id = main.transaction_id`,
  main_table_alias: 'main',
  defaultOrderBy: [],
  lowLevelOrderBy: [],
  filter_period_config: {
    hidden: true,
  },

  column_configs: [
    {
      column: 'main__status',
      query: 'main.status',
      label: 'Status',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.STATUS,
    },
    {
      column: 'main__code',
      query: 'main.code',
      label: 'Kode',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'main__request_date',
      query: 'main.request_date',
      label: 'Tgl. Permintaan',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.DATE_TIMESTAMP,
      date_format: 'DD/MM/YYYY',
    },
    {
      column: 'main__refund_date',
      query: 'main.refund_date',
      label: 'Tgl. Refund',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.DATE_TIMESTAMP,
      date_format: 'DD/MM/YYYY',
    },
    {
      column: 'main__type',
      query: 'main.type',
      label: 'Tipe Refund',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'tr__invoice_code',
      query: 'tr.invoice_code',
      label: 'Kode Settlement',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'tr__settlement_date',
      query: 'tr.settlement_date',
      label: 'Tgl. Settlement',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.DATE_EPOCH,
      date_format: 'DD/MM/YYYY',
    },
    {
      column: 'tr__payment_total',
      query: 'tr.payment_total',
      label: 'Total Pembayaran',
      type: DATA_TYPE.MEASURE,
      format: DATA_FORMAT.CURRENCY,
    },
    {
      column: 'main__refund_total',
      query: 'main.refund_total',
      label: 'Total Pengembalian',
      type: DATA_TYPE.MEASURE,
      format: DATA_FORMAT.CURRENCY,
    },
    {
      column: 'tr__payment_type',
      query: 'tr.payment_type',
      label: 'Tipe Pembayaran',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'tr__payment_type_method_name',
      query: 'tr.payment_type_method_name',
      label: 'Bank',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'tr__customer_name',
      query: 'tr.customer_name',
      label: 'Kontak',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'tr__customer_phone',
      query: 'tr.customer_phone',
      label: 'Telepon',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'tr__customer_description',
      query: 'tr.customer_description',
      label: 'Deskripsi',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'main__creator_name',
      query: 'main.creator_name',
      label: 'Dibuat Oleh',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.TEXT,
    },
    {
      column: 'main__updated_at',
      query: 'main.updated_at',
      label: 'Tgl. Update',
      type: DATA_TYPE.DIMENSION,
      format: DATA_FORMAT.DATE_EPOCH,
    },
  ],
  filter_configs: [],
};
